UniqueSaturatedInto

Trait UniqueSaturatedInto 

pub trait UniqueSaturatedInto<T: Sized>: Sized {
    // Required method
    fn unique_saturated_into(self) -> T;
}
Just like Into except that if the source value is too big to fit into the destination type then it’ll saturate the destination.

Required Methods

fn unique_saturated_into(self) -> T

Consume self to return an equivalent value of T.

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors