pub trait Normalizable<T> {
// Required method
fn normalize(&self, targeted_sum: T) -> Result<Vec<T>, &'static str>;
}Expand description
A collection-like object that is made of values of type T and can normalize its individual
values around a centric point.
Note that the order of items in the collection may affect the result.
Required Methods
Dyn Compatibility
This trait is dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".